#!/usr/bin/env python
# -*- Mode: python; encoding: utf-8 -*-

## XChat OSD song notification
## (C) 2004 Gustavo J. A. M. Carneiro <gustavo@users.sf.net>
## Some code copied from xchatrb.py (xchat plugin), (c) Gustavo
## Carneiro and G-LiTe / <stephan@wilkogazu.nl>
## GPL license applies, blah blah

## More recent changes in main ChangeLog...
## Changes in version 0.3
## - Ditch gosd, require gnome-osd

## Changes in version 0.2
## - Thinner and less ugly border (Gustavo)

## Version: 0.1 (first release)

## Configuration variables:

SHOW_FULL_INFO = True



import pygtk; pygtk.require("2.0")
import CORBA
import bonobo.activation
import string
import bonobo
import gobject
from xml.sax.saxutils import escape
from gnomeosd import gnome_osd_conf
import gettext
import gnome.ui
import sys

try:
    import dbus
    if getattr(dbus, "version", (0,0,0)) >= (0,41,0):
        import dbus.glib
except:
    pass

gettext.install("gnome-osd", gnome_osd_conf.datadir + "/locale")

# missing consts from the typelib
_ACTIVATION_FLAG_NO_LOCAL = 1<<0;      # No shared libraries
_ACTIVATION_FLAG_PRIVATE = 1<<1;       # start a new server and don't register it
_ACTIVATION_FLAG_EXISTING_ONLY = 1<<2; # don't start the server if not started


_rhythmbox = None
def get_rhythmbox():
    global _rhythmbox

    def get_new(): return bonobo.activation.activate(
	"repo_ids.has('IDL:GNOME/Rhythmbox:1.0')",
	[], _ACTIVATION_FLAG_EXISTING_ONLY)

    if _rhythmbox is None:
	_rhythmbox = get_new()
    else:
	# Check if the cached reference points to a server that is
	# still alive, otherwise ask bonobo-activation for a new fresh
	# reference.
	try:
	    # basically this means: _rhythmbox.ping()
	    _rhythmbox.ref()
	    _rhythmbox.unref()
	except CORBA.COMM_FAILURE:
	    _rhythmbox = get_new()
    return _rhythmbox

class SongInfo(object):
    __slots__ = ['title', 'album', 'artist', 'duration', 'track_number']


def get_psong():
    rb = get_rhythmbox()
    if rb is None:
	print "Rhythmbox is not running"
	return None
    pb = rb.getPlayerProperties()
    info = pb.getValue("song").value()
    pb.unref()
    if info is None:
        print "Rhythmbox is not playing anything"
        return None
    if not info.title:
        return "(untitled song)"
    return format_psong(info)

def format_psong(info):
    #  --- duration ---
    duration = info.duration
    if duration is None or duration == 0:
	duration_str = None
    else:
	if duration == -1:
	    print "Rhythmbox is not playing anything"
	    return None
	else:
	    hours    = duration // 3600
	    duration = duration % 3600
	    minutes  = duration // 60
	    duration = duration % 60
	    duration_str = str(minutes) + ":" + str(duration).zfill(2)
	    if hours == 0:
		duration_str = '%s:%s' % (str(minutes), str(duration).zfill(2))
	    else:
		duration_str = '%s:%s:%s' % (str(hours), str(minutes).zfill(2),
					     str(duration).zfill(2))
    # --- title ---
    title = escape(info.title)
    if title.endswith('.mp3') or title.endswith('.ogg'):
        title = title[:-4]
    msg = "♪ <span foreground='yellow'>%s</span>" % title
    if duration_str is not None:
        msg += " <span size='smaller'>(%s)</span> ♪" % duration_str
    else:
        msg += " ♪"

    extra_info = []
    # --- artist ---
    if info.artist:
        extra_info.append("<span size='smaller' style='italic'>%s</span>"
                          " <span foreground='yellow'>%s</span>" %
                          (_("Artist"), escape(info.artist)))

    # --- album ---
    if info.album:
        extra_info.append("<span size='smaller' style='italic'>%s</span>"
                          " <span foreground='yellow'>%s</span>" %
                          (_("Album"), escape(info.album)))
        if info.track_number != -1:
            extra_info.append("<span size='smaller' style='italic'>%s</span>"
                              " <span foreground='yellow'>%i</span>" %
                              (_("Track"), info.track_number))
    if SHOW_FULL_INFO and extra_info:
        return "\n".join((msg, "<span size='smaller'>%s</span>" % "   ".join(extra_info)))
    else:
        return msg

_osd = None
def get_osd():
    global _osd
    if _osd is not None:
        ser, cli = _osd
        try:
            # "ping" it
            ser.ref()
            ser.unref()
            return cli
        except:
            _osd = None
            print "OSD server died!"
    if _osd is None:
        osd = bonobo.get_object("OAFIID:GNOME_OSD", "IDL:Bonobo/Application:1.0")
        osd.ref() # AppClient steals one reference
        cli = bonobo.AppClient(osd)
        _osd = (osd, cli)
        return cli

def rb_properties_cb(listener, event, foo, *ev):
    osd = get_osd()
    song = get_psong()
    if isinstance(song, unicode):
        song = song.encode("utf-8")
    osd.msg_send("show-full",
                 ("<message id='rhythmbox' hide_timeout='10000'>" + song + "</message>",))

def monitor_rhythmbox(*args):
    global listener
    rb = get_rhythmbox()
    if rb is not None:
        if listener is not None:
            bonobo.event_source_client_remove_listener(
                rb.getPlayerProperties(), listener)
        listener = bonobo.event_source_client_add_listener(
            rb.getPlayerProperties(), rb_properties_cb,
            "Bonobo/Property:change:song")

def die_cb(cli):
    print "Die!!"
    global listener
    rb = get_rhythmbox()
    if rb is not None:
        if listener is not None:
            bonobo.event_source_client_remove_listener(
                rb.getPlayerProperties(), listener)
    bonobo.main_quit()
    print "OK, quitting..."

gnome.program_init("rbosd.py", "")
cli = gnome.ui.master_client()
cli.connect("die", die_cb)
cli.set_restart_command(1, sys.argv[:1])

listener = bonobo.event_source_client_add_listener(
    bonobo.activation.activate("iid == 'OAFIID:Bonobo_Activation_EventSource'"),
    monitor_rhythmbox, "Bonobo/ObjectDirectory:activation:register")

monitor_rhythmbox()

def evolution_new_mail_callback(arg):
    if not arg.endswith("INBOX"):
        return
    osd = get_osd()
    osd.msg_send("show-full",
                 ("<message id='newmail' hide_timeout='3000'>You Have Mail</message>",))

def muine_song_change_cb(arg):
    d = dict((x.strip() for x in  s1.split(":", 1)) for s1 in arg.split("\n"))
    info = SongInfo()
    info.title = d['title']
    info.album = d['album']
    info.artist = d['artist']
    info.duration = int(d['duration'])
    info.track_number = int(d['track_number'])
    song = format_psong(info)
    osd = get_osd()
    if isinstance(song, unicode):
        song = song.encode("utf-8")
    osd.msg_send("show-full",
                 ("<message id='muine' hide_timeout='10000'>" + song + "</message>",))


def monitor_dbus_apps():
    bus = dbus.SessionBus()
    bus.add_signal_receiver(evolution_new_mail_callback, "Newmail",
                            "org.gnome.evolution.mail.dbus.Signal")
    bus.add_signal_receiver(muine_song_change_cb, "SongChanged",
                            "org.gnome.Muine.Player")

try:
    monitor_dbus_apps()
except:
    print >> sys.stderr, "Unable to monitor D-BUS applications"

bonobo.main()
